#include <GL/glut.h>
#include "externs.h"

/***** color scheme used in this scene *****/
#define CS390_POINTS_COLOR      0.0, 0.5, 0.0           /* dark green */
#define CS390_CURVE_COLOR       1.0, 0.0, 0.0           /* red */

#define MAX_NUM_POINTS          20
#define MAX_NUM_KNOTS           40

/***** function declaration *****/
void draw_bspline_curve(int, int, int, GLfloat [], GLfloat [][3]);
void draw_disk(GLfloat, GLfloat, GLfloat, GLfloat);

/***** global variable *****/
GLfloat u = 0.0;          /** this value get changed from 0.0 to 1.0 by pressing
                         ** "left" and "right" arrows on keyboard. To control
                         ** the changing step, modify the "cs390_U_STEP" value
                         ** in "config.h" file. **/


/***** example curve data *****/
int degree = 3;         /** you should read in curve degree  */
int num_points = 7;     /** number of control points */
int num_knots = 11;     /** number of knots from your data file */

GLfloat knot_vector[MAX_NUM_KNOTS] = {0.0, 0.0, 0.0, 0.0,
                                        0.5, 0.5, 0.5,
                                        1.0, 1.0, 1.0, 1.0};

GLfloat points_data[MAX_NUM_POINTS][3] = {      { 50.0, 50.0, 50.0},
                                                { 50.0, 50.0,  0.0},
                                                { 50.0,  0.0,  0.0},
                                                {  0.0,  0.0,  0.0},
                                                {-50.0,  0.0,  0.0},
                                                {-50.0,-50.0,  0.0},
                                                {-50.0,-50.0,-50.0}

                                         };

/*** Your own rendering operation goes here ***/
void cs390_draw()
{
    draw_bspline_curve(num_knots,
                        num_points,
                        degree,
                        knot_vector,
                        points_data);
    /***** other rendering code goes here *****/
    glColor3f(1.0, 1.0, 0.0);
    draw_disk(0.0, 0.0, 0.0, 4.0);
}


/*** Your initialization goes here ***/
/*** for example, read the data from the file ***/
/*** for a B-spline curve, you need to read in the number of control points,
 *** the number of knots in knot vector, the degree of the curve, along with
 *** the knot vector and the control points ***/
/*** "argc", "argv" are actual command line arguments passed in ***/
void cs390_init(int argc, char **argv)
{
}


/*****************************************************************************/
/*** Primitivie which draws a B-spline curve and its control points ***/
/*** Param:     m : the number of knots in knot vector
 ***            n : the number of control points
 ***            p : the curve degree
 ***            knot_v : the knot vector
 ***            points_v : the control points
 *** Note:
 ***    Special relationship among 'n', 'm' and 'p' has to be satisfied to
 ***    define a B-spline curve. i.e.
 ***            m = n + p + 1
 ***    Make sure your data fit this requriement.
 *****************************************************************************/
void draw_bspline_curve(int m, int n, int p,
                        GLfloat knot_v[], GLfloat points_v[][3])
{
    int i;

    /* draw the control points first */
    glColor3f(CS390_POINTS_COLOR);
    glPointSize(8);
    glBegin(GL_POINTS);
        for(i = 0; i < n; i++) {
            glVertex3fv(&points_v[i][0]);
        }
    glEnd();

    /* draw control polygon */
    glColor3f(CS390_POINTS_COLOR);
    glLineWidth(4);
    glBegin(GL_LINE_STRIP);
    for (i = 0; i < n; i++)
         glVertex3fv(&points_v[i][0]);
    glEnd();
        
    /* draw the B-spline curve */
    if(m != n + p + 1) {        /* redundant check */
        printf("Relation between #points, #knots and degree is");
        printf("not satisfied!\n Can't draw B-spline curve1\n");
        return;
    }
    glColor3f(CS390_CURVE_COLOR);
    glLineWidth(2);
    if(n >= 2) {
    	   glShadeModel(GL_FLAT);
        glEnable(GL_MAP1_VERTEX_3);
        gluBeginCurve(theNurbs);
        gluNurbsCurve(theNurbs,
                        m,
                        &knot_v[0],
                        3,
                        &points_v[0][0],
                        p + 1,
                        GL_MAP1_VERTEX_3);
        gluEndCurve(theNurbs);
    }
}

void draw_disk(GLfloat x, GLfloat y, GLfloat z, GLfloat size)
{
    glLineWidth(1);
    glPushMatrix();
        glTranslated(x, y, z);
        gluDisk(theDisk, 0.0, size, 16, 2);
    glPopMatrix();
}

